package net.asukaze.cassava.core.command;

import java.util.List;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;

public class DeleteRowCommand implements Command {
	private ICassavaTableModel model;
	private ICassavaViewModel viewModel;
	private ICassavaTableModel deleted;
	private int topRowIndex;
	private int bottomRowIndex;

	public DeleteRowCommand(ICassavaTableModel model,
			ICassavaViewModel viewModel, int topRowIndex, int bottomRowIndex) {
		this.model = model;
		this.viewModel = viewModel;
		this.deleted = null;
		this.topRowIndex = topRowIndex;
		this.bottomRowIndex = bottomRowIndex;
	}

	public boolean canUndo() {
		return true;
	}

	public Status run() throws CommandException {
		if (topRowIndex <= bottomRowIndex) {
			deleted = ModelUtil.copy(model, new Selection(0, topRowIndex, model
					.getColumnCount() - 1, bottomRowIndex));
			for (int y = bottomRowIndex; y >= topRowIndex; y--) {
				model.deleteRow(y);
				List<Integer> rowHeights = viewModel.getRowHeights();
				if (rowHeights.size() >= y) {
					rowHeights.remove(y);
				}
			}
			int currentRow = viewModel.getCurrentRow();
			if (currentRow >= topRowIndex && currentRow <= bottomRowIndex) {
				currentRow = topRowIndex;
				viewModel.setCurrentRow(currentRow);
			} else if (currentRow > bottomRowIndex) {
				currentRow -= (bottomRowIndex - topRowIndex + 1);
				viewModel.setCurrentRow(currentRow);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}

	public Status undo() throws CommandException {
		if (topRowIndex <= bottomRowIndex && deleted != null) {
			for (int y = topRowIndex; y <= bottomRowIndex; y++) {
				model.insertRow(y);
			}
			ModelUtil.paste(model, new Selection(0, topRowIndex, model
					.getColumnCount() - 1, bottomRowIndex), deleted);
			deleted = null;
			int currentRow = viewModel.getCurrentRow();
			if (currentRow >= topRowIndex) {
				currentRow += (bottomRowIndex - topRowIndex + 1);
				viewModel.setCurrentRow(currentRow);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}

}
