package net.asukaze.cassava.core.model;

import java.util.ArrayList;
import java.util.List;

public class CassavaViewModel implements ICassavaViewModel {
	private int currentRow = 0;
	private int currentCol = 0;
	private int visibleFirstRow = 1;
	private int visibleFirstCol = 1;
	private int fixRowCount = 0;
	private int fixColCount = 0;
	private Selection selection = null;
	private List<Integer> colWidths = new ArrayList<Integer>();
	private List<Integer> rowHeights = new ArrayList<Integer>();
	private int defaultColWidth = 32;
	private int defaultRowHeight = 24;

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setCurrentRow(int)
	 */
	public void setCurrentRow(int currentRow) {
		this.currentRow = currentRow;
		if (!getSelection().isSelected(currentCol, currentRow)) {
			selection = Selection.create(currentCol, currentRow);
		}
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getCurrentRow()
	 */
	public int getCurrentRow() {
		return currentRow;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setCurrentCol(int)
	 */
	public void setCurrentCol(int currentCol) {
		this.currentCol = currentCol;
		if (!getSelection().isSelected(currentCol, currentRow)) {
			selection = Selection.create(currentCol, currentRow);
		}
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getCurrentCol()
	 */
	public int getCurrentCol() {
		return currentCol;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setCurrentRowCol(int, int)
	 */
	public void setCurrentRowCol(int currentRow, int currentCol) {
		setCurrentRow(currentRow);
		setCurrentCol(currentCol);
		setSelection(Selection.create(currentCol, currentRow));
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setVisibleFirstRow(int)
	 */
	public void setVisibleFirstRow(int visibleFirstRow) {
		this.visibleFirstRow = visibleFirstRow;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getVisibleFirstRow()
	 */
	public int getVisibleFirstRow() {
		return visibleFirstRow;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setVisibleFirstCol(int)
	 */
	public void setVisibleFirstCol(int visibleFirstCol) {
		this.visibleFirstCol = visibleFirstCol;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getVisibleFirstCol()
	 */
	public int getVisibleFirstCol() {
		return visibleFirstCol;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setFixRowCount(int)
	 */
	public void setFixRowCount(int fixRowCount) {
		this.fixRowCount = fixRowCount;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getFixRowCount()
	 */
	public int getFixRowCount() {
		return fixRowCount;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setFixColCount(int)
	 */
	public void setFixColCount(int fixColCount) {
		this.fixColCount = fixColCount;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getFixColCount()
	 */
	public int getFixColCount() {
		return fixColCount;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setSelection(net.asukaze.cassava.core.model.Selection)
	 */
	public void setSelection(Selection selection) {
		this.selection = selection;
		if (selection != null) {
			if (currentRow < selection.top || currentRow > selection.bottom) {
				currentRow = selection.bottom;
			}
			if (currentCol < selection.left || currentCol > selection.right) {
				currentCol = selection.right;
			}
		}
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getSelection()
	 */
	public Selection getSelection() {
		if (selection == null) {
			selection = Selection.create(currentCol, currentRow);
		}
		return selection;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setColWidths(java.util.List)
	 */
	public void setColWidths(List<Integer> colWidths) {
		this.colWidths = colWidths;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getColWidths()
	 */
	public List<Integer> getColWidths() {
		return colWidths;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setRowHeights(java.util.List)
	 */
	public void setRowHeights(List<Integer> rowHeights) {
		this.rowHeights = rowHeights;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getRowHeights()
	 */
	public List<Integer> getRowHeights() {
		return rowHeights;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setDefaultColWidth(int)
	 */
	public void setDefaultColWidth(int defaultColWidth) {
		this.defaultColWidth = defaultColWidth;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getDefaultColWidth()
	 */
	public int getDefaultColWidth() {
		return defaultColWidth;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setDefaultRowHeight(int)
	 */
	public void setDefaultRowHeight(int defaultRowHeight) {
		this.defaultRowHeight = defaultRowHeight;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getDefaultRowHeight()
	 */
	public int getDefaultRowHeight() {
		return defaultRowHeight;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setColWidth(int, int)
	 */
	public void setColWidth(int colIndex, int colWidth) {
		while (colIndex >= colWidths.size()) {
			colWidths.add(-1);
		}
		colWidths.set(colIndex, colWidth);
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getColWidth(int)
	 */
	public int getColWidth(int colIndex) {
		if (colIndex < colWidths.size()) {
			int width = colWidths.get(colIndex);
			if (width >= 0) {
				return width;
			}
		}
		return defaultColWidth;
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#setRowHeight(int, int)
	 */
	public void setRowHeight(int rowIndex, int rowHeight) {
		while (rowHeight >= rowHeights.size()) {
			rowHeights.add(-1);
		}
		rowHeights.set(rowIndex, rowHeight);
	}

	/* (non-Javadoc)
	 * @see net.asukaze.cassava.core.model.ICassavaViewModel#getRowHeight(int)
	 */
	public int getRowHeight(int rowIndex) {
		if (rowIndex < rowHeights.size()) {
			int height = rowHeights.get(rowIndex);
			if (height >= 0) {
				return height;
			}
		}
		return defaultRowHeight;
	}
}