package net.asukaze.cassava.core.util;

import net.asukaze.cassava.core.model.CassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;

public class ModelUtil {
	private enum Behavior {
		CUT, COPY, PASTE
	}

	public static final String EMPTY = ""; //$NON-NLS-1$

	private static ICassavaTableModel copy(ICassavaTableModel model,
			Selection selection, Behavior behavior, ICassavaTableModel data,
			boolean repeat) {
		ICassavaTableModel result = new CassavaTableModel();
		int rows = selection.bottom - selection.top + 1;
		int cols = selection.right - selection.left + 1;
		int top = selection.top;
		int left = selection.left;
		for (int y = 0; y < rows; y++) {
			for (int x = 0; x < cols; x++) {
				String value = model.getValueAt(y + top, x + left);
				result.setValueAt(value, y, x);
				if (behavior == Behavior.CUT) {
					model.setValueAt(EMPTY, y + top, x + left);
				} else if (behavior == Behavior.PASTE) {
					String d;
					if (repeat) {
						d = data.getValueAt(y % data.getRowCount(), x
								% data.getColumnCount());
					} else {
						d = data.getValueAt(y, x);
					}
					model.setValueAt(d, y + top, x + left);
				}
			}
		}
		return result;
	}

	public static ICassavaTableModel copy(ICassavaTableModel model,
			Selection selection) {
		return copy(model, selection, Behavior.COPY, null, false);
	}

	public static ICassavaTableModel cut(ICassavaTableModel model,
			Selection selection) {
		return copy(model, selection, Behavior.CUT, null, false);
	}

	/**
	 * model ̑I͈͂ data \tÃf[^̃Rs[Ԃ܂.<br />
	 * I͈͂ data 傫ꍇAZ̓NA܂.<br />
	 * data I͈͂傫ꍇAI͈͊Õf[^͖܂.
	 * 
	 * @param model
	 *            \t̃f
	 * @param selection
	 *            I͈
	 * @param data
	 *            \tf[^
	 * @return XI͈͂ɂf[^̃Rs[
	 */
	public static ICassavaTableModel paste(ICassavaTableModel model,
			Selection selection, ICassavaTableModel data) {
		return copy(model, selection, Behavior.PASTE, data, false);
	}

	public static ICassavaTableModel paste(ICassavaTableModel model,
			Selection selection, ICassavaTableModel data, boolean repeat) {
		return copy(model, selection, Behavior.PASTE, data, repeat);
	}

	public static void insertCellsRight(ICassavaTableModel model,
			Selection selection) {
		int deltaY = selection.bottom - selection.top + 1;
		int deltaX = selection.right - selection.left + 1;
		model.insertCellsRight(selection.top, selection.left, deltaY, deltaX);
	}

	public static void insertCellsDown(ICassavaTableModel model,
			Selection selection) {
		int deltaY = selection.bottom - selection.top + 1;
		for (int y = model.getRowCount() - 1; y >= selection.top; y--) {
			for (int x = selection.right; x >= selection.left; x--) {
				String value = model.getValueAt(y, x);
				model.setValueAt(value, y + deltaY, x);
				model.setValueAt(EMPTY, y, x);
			}
		}
	}

	public static void deleteCellsLeft(ICassavaTableModel model,
			Selection selection) {
		int deltaY = selection.bottom - selection.top + 1;
		int deltaX = selection.right - selection.left + 1;
		model.deleteCellsLeft(selection.top, selection.left, deltaY, deltaX);
	}

	public static void deleteCellsUp(ICassavaTableModel model,
			Selection selection) {
		int deltaY = selection.bottom - selection.top + 1;
		for (int y = selection.top; y <= model.getRowCount() - 1; y++) {
			for (int x = selection.left; x <= selection.right; x++) {
				String value = model.getValueAt(y + deltaY, x);
				model.setValueAt(value, y, x);
			}
		}
	}
}
