package net.asukaze.cassava.io.command;

import java.io.IOException;
import java.io.Reader;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.core.util.DataFormatSetting;
import net.asukaze.cassava.io.util.CsvReader;

public class OpenFileCommand implements Command {
	private ICassavaTableModel model;
	private ICassavaViewModel viewModel;
	private Reader reader;
	private String filename;

	public OpenFileCommand(ICassavaTableModel model, ICassavaViewModel viewModel,
			Reader reader, String filename) {
		this.model = model;
		this.viewModel = viewModel;
		this.reader = reader;
		this.filename = filename;
	}

	public boolean canUndo() {
		return false;
	}

	public Status run() throws CommandException {
		DataFormat dataFormat = DataFormatSetting.INSTANCE
				.getDataFormatForFilename(filename);
		CsvReader csvr = new CsvReader(dataFormat, reader);
		try {
			csvr.readTable(model);
			model.setFilename(filename);
			viewModel.setCurrentRowCol(1, 1);
		} catch (IOException ex) {
			throw new CommandException(ex);
		} finally {
			try {
				csvr.close();
				model = null;
				viewModel = null;
				reader = null;
			} catch (IOException ex) {
				throw new CommandException(ex);
			}
		}
		return Status.CHANGED;
	}

	public Status undo() {
		return Status.FAIL;
	}

}
