package net.asukaze.cassava.io.command;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;

public class PasteCommand implements Command {
	public enum Behavior {
		OVERRAPPED, REPEAT, DATASIZE, MOVERIGHT, MOVEDOWN
	}

	private ICassavaTableModel tableModel;
	private ICassavaTableModel data;
	private Behavior behavior;
	private Selection selection;
	private ICassavaTableModel old;
	private int rowCount;
	private int columnCount;

	public PasteCommand(ICassavaTableModel tableModel,
			Selection selection, ICassavaTableModel data,
			Behavior behavior) {
		this.tableModel = tableModel;
		this.data = data;
		this.behavior = behavior;
		this.selection = selection;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	@Override
	public Status run() throws CommandException {
		this.rowCount = tableModel.getRowCount();
		this.columnCount = tableModel.getColumnCount();
		switch (behavior) {
		case OVERRAPPED:
			selection.bottom = Math.min(selection.bottom, selection.top
					+ data.getRowCount() - 1);
			selection.right = Math.min(selection.right, selection.left
					+ data.getColumnCount() - 1);
			old = ModelUtil.paste(tableModel, selection, data);
			break;
		case REPEAT:
			old = ModelUtil.paste(tableModel, selection, data, true);
			break;
		case DATASIZE:
			selection.bottom = selection.top + data.getRowCount() - 1;
			selection.right = selection.left + data.getColumnCount() - 1;
			old = ModelUtil.paste(tableModel, selection, data);
			break;
		case MOVERIGHT:
			selection.bottom = selection.top + data.getRowCount() - 1;
			selection.right = selection.left + data.getColumnCount() - 1;
			old = null;
			ModelUtil.insertCellsRight(tableModel, selection);
			ModelUtil.paste(tableModel, selection, data);
			break;
		case MOVEDOWN:
			selection.bottom = selection.top + data.getRowCount() - 1;
			selection.right = selection.left + data.getColumnCount() - 1;
			old = null;
			ModelUtil.insertCellsDown(tableModel, selection);
			ModelUtil.paste(tableModel, selection, data);
			break;
		}
		data = null;
		return Status.CHANGED;
	}

	@Override
	public Status undo() throws CommandException {
		if (selection != null) {
			switch (behavior) {
			case OVERRAPPED:
			case REPEAT:
				data = ModelUtil.paste(tableModel, selection, old);
				old = null;
				return Status.CHANGED;
			case DATASIZE:
				data = ModelUtil.paste(tableModel, selection, old);
				old = null;
				tableModel.setRowCount(rowCount);
				tableModel.setColumnCount(columnCount);
				return Status.CHANGED;
			case MOVERIGHT:
				data = ModelUtil.cut(tableModel, selection);
				ModelUtil.deleteCellsLeft(tableModel, selection);
				tableModel.setRowCount(rowCount);
				tableModel.setColumnCount(columnCount);
				return Status.CHANGED;
			case MOVEDOWN:
				data = ModelUtil.cut(tableModel, selection);
				ModelUtil.deleteCellsUp(tableModel, selection);
				tableModel.setRowCount(rowCount);
				tableModel.setColumnCount(columnCount);
				return Status.CHANGED;
			}
		}
		return Status.NO_CHANGE;
	}
}
