package net.asukaze.cassava.io.util;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.StringReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;

public class ClipboardUtil {
	private static Object getClipboardService() throws ClassNotFoundException,
			SecurityException, NoSuchMethodException, IllegalArgumentException,
			IllegalAccessException, InvocationTargetException {
		Class<?> serviceManager = Class.forName("javax.jnlp.ServiceManager"); //$NON-NLS-1$
		Method lookup = serviceManager.getMethod("lookup", String.class); //$NON-NLS-1$
		Object clipboardService = lookup.invoke(null,
				"javax.jnlp.ClipboardService"); //$NON-NLS-1$
		return clipboardService;
	}

	public static void copy(String value) {
		StringSelection selection = new StringSelection(value);
		boolean nojnlp = true;
		try {
			Object clipboardService = getClipboardService();
			Method setContents = clipboardService.getClass().getMethod(
					"setContents", Transferable.class); //$NON-NLS-1$
			setContents.invoke(clipboardService, selection);
			nojnlp = false;
		} catch (Exception e) {
			nojnlp = true;
		}
		if (nojnlp) {
			Clipboard clipboard = Toolkit.getDefaultToolkit()
					.getSystemClipboard();
			clipboard.setContents(selection, selection);
		}
	}

	public static String paste() throws IOException, UnsupportedFlavorException {
		boolean nojnlp = true;
		Transferable contents = null;
		try {
			Object clipboardService = getClipboardService();
			Method getContents = clipboardService.getClass().getMethod(
					"getContents"); //$NON-NLS-1$
			contents = (Transferable) getContents.invoke(clipboardService);
			nojnlp = false;
		} catch (Exception e) {
			nojnlp = true;
		}
		if (nojnlp) {
			Clipboard clipboard = Toolkit.getDefaultToolkit()
					.getSystemClipboard();
			contents = clipboard.getContents(null);
		}
		return (String) contents.getTransferData(DataFlavor.stringFlavor);
	}

	public static ICassavaTableModel pasteAsTableModel(DataFormat dataFormat)
			throws IOException, UnsupportedFlavorException {
		String value = ClipboardUtil.paste();
		StringReader sr = new StringReader(value);
		CsvReader csvr = new CsvReader(dataFormat, sr);
		try {
			return csvr.readTable();
		} finally {
			csvr.close();
			sr.close();
		}
	}
}
