package net.asukaze.cassava.io.util;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.util.DataFormatSetting;

import org.junit.Test;

public class CsvReaderTest {
	@Test
	public void testReadTable1() throws IOException {
		InputStream is = CsvReaderTest.class.getResourceAsStream("test1.txt"); //$NON-NLS-1$
		CsvReader reader = new CsvReader(DataFormatSetting.INSTANCE
				.getDefault(), new InputStreamReader(is));
		ICassavaTableModel model = reader.readTable();
		reader.close();
		assertEquals(3, model.getColumnCount());
		assertEquals("AAA", model.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("BBB", model.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("CCC", model.getValueAt(0, 2)); //$NON-NLS-1$
		assertEquals("DDD", model.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("EEE", model.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("FFF", model.getValueAt(1, 2)); //$NON-NLS-1$
		assertEquals("GGG", model.getValueAt(2, 0)); //$NON-NLS-1$
		assertEquals("HHH", model.getValueAt(2, 1)); //$NON-NLS-1$
		assertEquals("III", model.getValueAt(2, 2)); //$NON-NLS-1$
		assertEquals("JJJ", model.getValueAt(3, 0)); //$NON-NLS-1$
		assertEquals("KKK", model.getValueAt(3, 1)); //$NON-NLS-1$
		assertEquals("LLL", model.getValueAt(3, 2)); //$NON-NLS-1$
		assertEquals("MMM", model.getValueAt(4, 0)); //$NON-NLS-1$
		assertEquals("NNN", model.getValueAt(4, 1)); //$NON-NLS-1$
		assertEquals("OOO", model.getValueAt(4, 2)); //$NON-NLS-1$
		assertEquals("MMM", model.getValueAt(5, 0)); //$NON-NLS-1$
		assertEquals("NNN", model.getValueAt(5, 1)); //$NON-NLS-1$
		assertEquals("OOO", model.getValueAt(5, 2)); //$NON-NLS-1$
		assertEquals("P P P", model.getValueAt(6, 0)); //$NON-NLS-1$
		assertEquals("QQQ\nRRR\nSSS", model.getValueAt(6, 1)); //$NON-NLS-1$
		assertEquals("T,T,T", model.getValueAt(6, 2)); //$NON-NLS-1$
		assertEquals("UUU", model.getValueAt(7, 0)); //$NON-NLS-1$
		assertEquals("VVV", model.getValueAt(7, 1)); //$NON-NLS-1$
		assertEquals("WWW", model.getValueAt(7, 2)); //$NON-NLS-1$
		assertEquals("X\"X\"X", model.getValueAt(8, 0)); //$NON-NLS-1$
		assertEquals("Y\"Y\"Y", model.getValueAt(8, 1)); //$NON-NLS-1$
		assertEquals("Z\"Z\"Z", model.getValueAt(8, 2)); //$NON-NLS-1$
	}

	@Test
	public void testReadTable2() throws IOException {
		InputStream is = CsvReaderTest.class.getResourceAsStream("test2.txt"); //$NON-NLS-1$
		DataFormat dataFormat = new DataFormat(null, false, new String[0], "|",
				"-", "'", true, false);
		CsvReader reader = new CsvReader(dataFormat, new InputStreamReader(is));
		ICassavaTableModel model = reader.readTable();
		assertEquals("AAA", model.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("BBB", model.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("CCC", model.getValueAt(0, 2)); //$NON-NLS-1$
		assertEquals(4, model.getColumnCount(0));
		assertEquals("D'D'D", model.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("E'E'E", model.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("F'F'F", model.getValueAt(1, 2)); //$NON-NLS-1$
		assertEquals(4, model.getColumnCount(1));
		assertEquals("", model.getValueAt(2, 0)); //$NON-NLS-1$
		assertEquals(1, model.getColumnCount(2));
		assertEquals("GGG", model.getValueAt(3, 0)); //$NON-NLS-1$
		assertEquals("HHH", model.getValueAt(3, 1)); //$NON-NLS-1$
		assertEquals("III", model.getValueAt(3, 2)); //$NON-NLS-1$
		assertEquals(3, model.getColumnCount(3));
		assertEquals("J'J'J", model.getValueAt(4, 0)); //$NON-NLS-1$
		assertEquals("K'K'K", model.getValueAt(4, 1)); //$NON-NLS-1$
		assertEquals("L'L'L", model.getValueAt(4, 2)); //$NON-NLS-1$
		assertEquals(3, model.getColumnCount(4));
		assertEquals("", model.getValueAt(5, 0)); //$NON-NLS-1$
		assertEquals("", model.getValueAt(5, 1)); //$NON-NLS-1$
		assertEquals(2, model.getColumnCount(5));
		assertEquals("", model.getValueAt(6, 0)); //$NON-NLS-1$
		assertEquals(1, model.getColumnCount(6));
		assertEquals("\n", model.getValueAt(7, 0)); //$NON-NLS-1$
		assertEquals(1, model.getColumnCount(7));
		assertEquals(0, model.getColumnCount(8));
		reader.finalize();
	}

	@Test
	public void testNext() throws IOException {
		InputStream is = CsvReaderTest.class.getResourceAsStream("test2.txt"); //$NON-NLS-1$
		DataFormat dataFormat = new DataFormat(null, false, new String[0], "-",
				"|", "'", false, false);
		CsvReader reader = new CsvReader(dataFormat, new InputStreamReader(is));
		assertEquals("AAA", reader.next()); //$NON-NLS-1$
		assertEquals("BBB", reader.next()); //$NON-NLS-1$
		assertEquals("CCC", reader.next()); //$NON-NLS-1$
		assertEquals("'D''D''D'", reader.next()); //$NON-NLS-1$
		assertEquals("'E''E''E'", reader.next()); //$NON-NLS-1$
		assertEquals("'F''F''F'", reader.next()); //$NON-NLS-1$
		assertEquals("", reader.next()); //$NON-NLS-1$
		assertEquals("GGG", reader.next()); //$NON-NLS-1$
		assertEquals("HHH", reader.next()); //$NON-NLS-1$
		assertEquals("III", reader.next()); //$NON-NLS-1$
		assertEquals("", reader.next()); //$NON-NLS-1$
		assertEquals("'J''J''J'", reader.next()); //$NON-NLS-1$
		assertEquals("'K''K''K'", reader.next()); //$NON-NLS-1$
		assertEquals("'L''L''L'", reader.next()); //$NON-NLS-1$
		assertEquals("", reader.next()); //$NON-NLS-1$
		assertEquals("", reader.next()); //$NON-NLS-1$
		assertEquals("", reader.next()); //$NON-NLS-1$
		assertEquals("", reader.next()); //$NON-NLS-1$
		assertEquals("'", reader.next()); //$NON-NLS-1$
		assertEquals(null, reader.next());
		assertEquals(null, reader.next());
		reader.close();
	}
}
