package net.asukaze.cassava.io.util;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.io.StringWriter;

import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.util.DataFormatSetting;

import org.junit.Test;

public class CsvWriterTest {
	private void assertModelEquals(ICassavaTableModel model1,
			ICassavaTableModel model2) {
		assertEquals(model1.getRowCount(), model2.getRowCount());
		assertEquals(model1.getColumnCount(), model2.getColumnCount());
		for (int y = 1; y <= model2.getRowCount(); y++) {
			for (int x = 1; x <= model2.getColumnCount(); x++) {
				assertEquals(model1.getValueAt(y, x), model2.getValueAt(y, x));
			}
		}
	}

	@Test
	public void testWriteTable1() throws IOException {
		InputStream is = CsvWriterTest.class.getResourceAsStream("test1.txt"); //$NON-NLS-1$
		DataFormat dataFormat = DataFormatSetting.INSTANCE.getDefault();

		CsvReader reader1 = new CsvReader(dataFormat, new InputStreamReader(is));
		ICassavaTableModel model1 = reader1.readTable();
		reader1.close();

		StringWriter sw = new StringWriter();
		CsvWriter writer = new CsvWriter(dataFormat, sw, "\r\n"); //$NON-NLS-1$
		writer.writeTable(model1);
		writer.close();

		System.out.println(sw.toString());
		System.out.println();

		CsvReader reader2 = new CsvReader(dataFormat, new StringReader(sw
				.toString()));
		ICassavaTableModel model2 = reader2.readTable();
		reader2.close();

		assertTrue(model1 != model2);
		assertModelEquals(model1, model2);
	}

	@Test
	public void testWriteTable2() throws IOException {
		InputStream is = CsvWriterTest.class.getResourceAsStream("test2.txt"); //$NON-NLS-1$
		DataFormat dataFormat = new DataFormat(null, false, new String[0], "|",
				"-", "'", true, false);
		CsvReader reader1 = new CsvReader(dataFormat, new InputStreamReader(is));
		ICassavaTableModel model1 = reader1.readTable();
		reader1.finalize();

		StringWriter sw = new StringWriter();
		CsvWriter writer = new CsvWriter(dataFormat, sw, "\r\n"); //$NON-NLS-1$
		writer.writeTable(model1);
		writer.close();

		System.out.println(sw.toString());
		System.out.println();

		CsvReader reader2 = new CsvReader(dataFormat, new StringReader(sw
				.toString()));
		ICassavaTableModel model2 = reader2.readTable();
		reader2.close();

		assertTrue(model1 != model2);
		assertModelEquals(model1, model2);
	}
}
