package net.asukaze.cassava.encoding.command;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.CassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;

public abstract class TransformCommand implements Command {
	private ICassavaTableModel model;
	private Selection selection;
	private ICassavaTableModel old;

	public TransformCommand(ICassavaTableModel model, Selection selection) {
		this.model = model;
		this.selection = selection;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	@Override
	public Status undo() throws CommandException {
		ModelUtil.paste(model, selection, old);
		return CHANGED;
	}

	@Override
	public Status run() throws CommandException {
		old = new CassavaTableModel();
		for (int y = selection.top; y <= selection.bottom; y++) {
			for (int x = selection.left; x <= selection.right; x++) {
				String oldValue = model.getValueAt(y, x);
				old.setValueAt(oldValue, y - selection.top, x - selection.left);
				model.setValueAt(transform(oldValue), y, x);
			}
		}
		return CHANGED;
	}

	abstract protected String transform(String value) throws CommandException;
}
