package net.asukaze.cassava.ui.action;

import javax.swing.JTextArea;
import javax.swing.table.TableCellEditor;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.io.command.PasteCommand;
import net.asukaze.cassava.io.util.ClipboardUtil;
import net.asukaze.cassava.ui.Log;
import net.asukaze.cassava.ui.dialog.PasteDialog;
import net.asukaze.cassava.ui.dialog.PasteDialog.Option;
import net.asukaze.cassava.ui.table.CassavaGrid;
import net.asukaze.cassava.ui.table.CassavaTableCellEditor;

public class PasteAction extends Action {
	private DataFormat format;
	private Selection afterSelection = null;

	protected void setFormat(DataFormat format) {
		this.format = format;
	}

	private Option getDefaultOption() {
		try {
			String value = getApplication().getProperties()
					.getProperty("Paste"); //$NON-NLS-1$
			if (value != null) {
				Integer id = Integer.parseInt(value);
				for (Option option : Option.getAll()) {
					if (option.getId() == id) {
						return option;
					}
				}
			}
		} catch (NumberFormatException ex) {
			Log.warn(ex);
		}
		return null;
	}

	private Option getNormalOption(CassavaGrid grid, Selection selection,
			ICassavaTableModel data, int clipRows, int clipColumns) {
		int selectedRows = selection.bottom - selection.top + 1;
		int selectedColumns = selection.right - selection.left + 1;
		boolean isRowSelected = (selectedRows == 1 && selectedColumns == grid
				.getColumnCount());
		boolean isColumnSelected = (selectedColumns == 1 && selectedRows == grid
				.getRowCount());

		if (isRowSelected && !isColumnSelected) {
			// sI͉Ɉړē\tBf[^ĂŜړB
			if (data.getColumnCount() < selectedColumns) {
				data.setColumnCount(selectedColumns);
			}
			return Option.MOVEDOWN;
		} else if (isColumnSelected && !isRowSelected) {
			// I͉EɈړē\tBf[^ĂŜړB
			if (data.getRowCount() < selectedRows) {
				data.setRowCount(selectedRows);
			}
			return Option.MOVERIGHT;
		} else if (clipRows == selectedRows && clipColumns == selectedColumns) {
			// I͈͂Ɠ傫Ȃ炻̂܂ܓ\t
			return Option.DATASIZE;
		} else {
			// I͈͂Ƒ傫قȂ΃_CAO\
			return PasteDialog.INSTANCE.show(selectedColumns, selectedRows,
					clipColumns, clipRows);
		}
	}

	@Override
	public Command run() throws Exception {
		afterSelection = null;
		CassavaGrid grid = getCassavaGrid();
		if (format == null) {
			format = getTableModel().getDataFormat();
		}
		ICassavaTableModel data = ClipboardUtil.pasteAsTableModel(format);
		int clipRows = data.getRowCount();
		int clipColumns = data.getColumnCount();
		if (!grid.isEditing() || clipRows > 1) {
			// e[u̓\t
			Selection selection = getViewModel().getSelection();
			// ܂͓\t@߂
			Option option = getDefaultOption();
			if (option == null) {
				// \t@uWv̏ꍇ
				option = getNormalOption(grid, selection, data, clipRows,
						clipColumns);
				if (option == null) {
					// _CAOŁuLZvsꂽꍇ
					return null;
				}
			}
			// ܂łœ\t܂̂ŁAoption ɏ] Command B
			if (option.equals(Option.OVERRAPPED)
					|| option.equals(Option.REPEAT)) {
				// ݂̑I͈͂ɑ΂ē\t
				return new PasteCommand(grid.getTableModel(), selection, data,
						option.toBehavior());
			} else if (option.equals(Option.DATASIZE)
					|| option.equals(Option.MOVERIGHT)
					|| option.equals(Option.MOVEDOWN)) {
				// Nbv{[hTCYœ\t
				this.afterSelection = new Selection(selection.left,
						selection.top, selection.left + clipColumns - 1,
						selection.top + clipRows - 1);
				return new PasteCommand(grid.getTableModel(), selection, data,
						option.toBehavior());
			} else {
				// ONECELL
				// eLXgƂ 1 Zɓ\t
				if (!grid.isEditing()) {
					grid.editCellAt(selection.bottom, selection.right);
				}
				// fall through...
			}
		}

		// eLXgƂ 1 Zɓ\t
		TableCellEditor cellEditor = grid.getCellEditor();
		if (cellEditor instanceof CassavaTableCellEditor) {
			JTextArea textArea = ((CassavaTableCellEditor) cellEditor)
					.getTextArea();
			textArea.paste();
		}
		return null;
	}

	@Override
	public void afterFirstRun() {
		if (afterSelection != null) {
			getViewModel().setSelection(afterSelection);
			afterSelection = null;
		}
	}
}
