package net.asukaze.cassava.ui.command;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;
import net.asukaze.cassava.core.util.ValueUtil;

public class MakeSequenceCommand implements Command {
	private ICassavaTableModel model;
	private Selection selection;
	private ICassavaTableModel old;

	public MakeSequenceCommand(ICassavaTableModel model, Selection selection) {
		this.model = model;
		this.selection = selection;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	private void copyFirstRow(int x) {
		String first = model.getValueAt(selection.top, x);
		for (int y = selection.top + 1; y <= selection.bottom; y++) {
			model.setValueAt(first, y, x);
		}
	}

	private void makeDoubleSequence(int x) {
		double first = Double.parseDouble(model.getValueAt(selection.top, x));
		double increment;
		String second = model.getValueAt(selection.top + 1, x);
		if (ValueUtil.isNumber(second)) {
			increment = Double.parseDouble(second) - first;
		} else {
			increment = 1.0;
		}
		for (int y = selection.top + 1; y <= selection.bottom; y++) {
			double value = first + increment * (y - selection.top);
			model.setValueAt(ValueUtil.toString(value), y, x);
		}
	}

	private void makeStringSequence(int x) {
		char[] arr = model.getValueAt(selection.top, x).toCharArray();
		for (int y = selection.top + 1; y <= selection.bottom; y++) {
			String next;
			int p = arr.length - 1;
			while (p >= 0 && arr[p] == '9') {
				arr[p] = '0';
				p--;
			}
			if (p == -1) {
				next = "1" + String.valueOf(arr);
				arr = next.toCharArray();
			} else if (arr[p] >= '0' && arr[p] < '9') {
				arr[p]++;
				next = String.valueOf(arr);
			} else {
				next = String.valueOf(arr, 0, p + 1) + "1"
						+ String.valueOf(arr, p + 1, arr.length - (p + 1));
				arr = next.toCharArray();
			}
			model.setValueAt(next, y, x);
		}

	}

	@Override
	public Status run() throws CommandException {
		old = ModelUtil.copy(model, selection);
		for (int x = selection.left; x <= selection.right; x++) {
			String first = model.getValueAt(selection.top, x);
			if (first.length() > 0) {
				if (ValueUtil.isNumber(first)) {
					makeDoubleSequence(x);
				} else {
					char lastChar = first.charAt(first.length() - 1);
					if (lastChar >= '0' && lastChar <= '9') {
						makeStringSequence(x);
					} else {
						copyFirstRow(x);
					}
				}
			} else {
				copyFirstRow(x);
			}
		}
		return CHANGED;
	}

	@Override
	public Status undo() throws CommandException {
		ModelUtil.paste(model, selection, old);
		old = null;
		return CHANGED;
	}
}
