package net.asukaze.cassava.ui.command;

import java.util.Comparator;
import java.util.SortedSet;
import java.util.TreeSet;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.CassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;
import net.asukaze.cassava.core.util.ValueUtil;

public class SortCommand implements Command {
	private static class Row {
		public String key;
		public int oldRow;
		public String[] data;

		public Row(String key, int oldRow, String[] data) {
			this.key = key;
			this.oldRow = oldRow;
			this.data = data;
		}

		@Override
		public String toString() {
			return "#Row:" + key;
		}
	}

	private ICassavaTableModel model;
	private int left;
	private int top;
	private int right;
	private int bottom;
	private int keyColumn;
	private boolean orderByDesc;
	private boolean number;
	private int[] oldRows = null;

	public SortCommand(ICassavaTableModel model, int left, int top, int right,
			int bottom, int keyColumn, boolean orderByDesc, boolean number) {
		this.model = model;
		this.left = left;
		this.top = top;
		this.right = right;
		this.bottom = bottom;
		this.keyColumn = keyColumn;
		this.orderByDesc = orderByDesc;
		this.number = number;
	}

	private class RowComparator implements Comparator<Row> {
		private int compareNoOrder(String str1, String str2) {
			if (number) {
				boolean n1 = ValueUtil.isNumber(str1);
				boolean n2 = ValueUtil.isNumber(str2);
				if (n1 && n2) {
					return Double.compare(Double.parseDouble(str1), Double
							.parseDouble(str2));
				} else if (n1) {
					return -1;
				} else if (n2) {
					return 1;
				}
				// else String#compareTo
			}
			return str1.compareTo(str2);
		}

		@Override
		public int compare(Row r1, Row r2) {
			int cmp = compareNoOrder(r1.key, r2.key);
			if (cmp == 0) {
				return (r1.oldRow - r2.oldRow);
			} else if (orderByDesc) {
				return -cmp;
			} else {
				return cmp;
			}
		}
	}

	private SortedSet<Row> createSortedSet() {
		SortedSet<Row> sortedSet = new TreeSet<Row>(new RowComparator());
		int cols = right - left + 1;
		for (int y = top; y <= bottom; y++) {
			String[] data = new String[cols];
			for (int i = 0; i < cols; i++) {
				data[i] = model.getValueAt(y, left + i);
			}
			String key = model.getCalculatedValueAt(y, keyColumn);
			sortedSet.add(new Row(key, y, data));
		}
		return sortedSet;
	}

	@Override
	public Status run() throws CommandException {
		SortedSet<Row> sortedSet = createSortedSet();
		Status status = Status.NO_CHANGE;
		oldRows = new int[sortedSet.size()];
		int y = top;
		int i = 0;
		for (Row row : sortedSet) {
			if (row.oldRow != y) {
				for (int j = 0; j < row.data.length; j++) {
					model.setValueAt(row.data[j], y, left + j);
				}
				status = Status.CHANGED;
			}
			oldRows[i] = row.oldRow;
			y++;
			i++;
		}
		return status;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	@Override
	public Status undo() throws CommandException {
		ICassavaTableModel oldModel = new CassavaTableModel();
		int cols = right - left + 1;
		for (int y = 0; y < oldRows.length; y++) {
			for (int x = 0; x < cols; x++) {
				String value = model.getValueAt(oldRows[y], left + x);
				oldModel.setValueAt(value, y, x);
			}
		}
		oldRows = null;
		Selection selection = new Selection(left, top, right, bottom);
		ModelUtil.paste(model, selection, oldModel);
		return Status.CHANGED;
	}
}
