package net.asukaze.cassava.ui.table;

import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.geom.Rectangle2D;

import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;

import net.asukaze.cassava.core.command.CommandStack;
import net.asukaze.cassava.core.model.CassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;

@SuppressWarnings("serial")
public class CassavaGrid extends JTable {
	private static final String LINE_SEPARATOR = "\n"; //$NON-NLS-1$
	private static final String EMPTY = ""; //$NON-NLS-1$

	private CassavaViewModel viewModel;
	private CommandStack commandStack = new CommandStack();

	/**
	 * This is the default constructor
	 */
	public CassavaGrid() {
		super();
		initialize(new CassavaTableModel());
	}

	public CassavaGrid(CassavaTableModel tableModel) {
		super();
		initialize(tableModel);
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize(CassavaTableModel tableModel) {
		this.setModel(tableModel);
		this.viewModel = new CassavaViewModel(this);
		CassavaTableColumnModel columnModel = new CassavaTableColumnModel();
		this.setColumnModel(columnModel);
		this.setTableHeader(new JTableHeader(columnModel));
		this.setDefaultRenderer(String.class, new CassavaTableCellRenderer());
		this.setDefaultEditor(String.class, new CassavaTableCellEditor());
	}

	public ICassavaViewModel getViewModel() {
		return viewModel;
	}

	public ICassavaTableModel getTableModel() {
		TableModel model = getModel();
		if (model instanceof CassavaTableModel) {
			return (ICassavaTableModel) model;
		}
		return null;
	}

	public CommandStack getCommandStack() {
		return commandStack;
	}

	@Override
	public boolean editCellAt(int row, int column) {
		setRowSelectionInterval(row, row);
		setColumnSelectionInterval(column, column);
		return super.editCellAt(row, column);
	}

	public Rectangle2D getStringBounds(String value) {
		Graphics g = getGraphics();
		FontMetrics fontMetrics = g.getFontMetrics();
		Rectangle2D rect = fontMetrics.getStringBounds(EMPTY, g);
		rect.setRect(0, 0, rect.getWidth(), 0);
		if (value.endsWith(LINE_SEPARATOR)) {
			// Ō̉šɕAsplit() ŏ񂪎̂h
			value = value + "\0";
		}
		for (String str : value.split(LINE_SEPARATOR)) {
			Rectangle2D r = fontMetrics.getStringBounds(str, g);
			double w = Math.max(rect.getWidth(), r.getWidth());
			double h = rect.getHeight() + fontMetrics.getHeight();
			rect.setRect(0, 0, w, h);
		}
		return rect;
	}
}
