package net.asukaze.cassava.ui.table;

import java.awt.Component;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.util.EventObject;

import javax.swing.AbstractCellEditor;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.TableCellEditor;

import net.asukaze.cassava.core.command.CellChangeCommand;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.ui.Log;

@SuppressWarnings("serial")
public class CassavaTableCellEditor extends AbstractCellEditor implements
		TableCellEditor {
	private JTextArea textArea;
	private JTable table;
	private int row;
	private int column;

	public CassavaTableCellEditor() {
		textArea = new JTextArea();
		textArea.addKeyListener(new TextKeyListener());
		textArea.setAutoscrolls(true);
	}

	public class TextKeyListener implements KeyListener {
		private void moveTo(int r, int c) {
			CassavaTableCellEditor.this.stopCellEditing();
			table.setRowSelectionInterval(r, r);
			table.setColumnSelectionInterval(c, c);
		}

		public void keyPressed(KeyEvent e) {
			int textLength = textArea.getText().length();
			if (e.getModifiers() == 0) {
				switch (e.getKeyCode()) {
				case KeyEvent.VK_UP:
					moveTo(row - 1, column);
					e.consume();
					break;
				case KeyEvent.VK_DOWN:
					moveTo(row + 1, column);
					e.consume();
					break;
				case KeyEvent.VK_LEFT:
					if (textArea.getSelectionStart() == 0) {
						if (textArea.getSelectionEnd() == 0
								|| textArea.getSelectionEnd() == textLength) {
							moveTo(row, column - 1);
							e.consume();
						}
					}
					break;
				case KeyEvent.VK_RIGHT:
					if (textArea.getSelectionEnd() == textLength) {
						if (textArea.getSelectionStart() == 0
								|| textArea.getSelectionStart() == textLength) {
							moveTo(row, column + 1);
							e.consume();
						}
					}
					break;
				case KeyEvent.VK_ENTER:
					/*
					 * if (textArea.getSelectionStart() == 0 &&
					 * textArea.getSelectionEnd() == textLength) {
					 * textArea.select(textLength, textLength); } else {
					 * textArea.selectAll(); }
					 */
					CassavaTableCellEditor.this.stopCellEditing();
					e.consume();
					break;
				}
			}
		}

		public void keyReleased(KeyEvent e) {
		}

		public void keyTyped(KeyEvent e) {
		}
	}

	@Override
	public Object getCellEditorValue() {
		return textArea.getText();
	}

	public JTextArea getTextArea() {
		return textArea;
	}

	@Override
	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {
		this.textArea.setText((String) value);
		this.table = table;
		this.row = row;
		this.column = column;
		return textArea;
	}

	public boolean isCellEditable(EventObject e) {
		if (e instanceof MouseEvent) {
			return ((MouseEvent) e).getClickCount() >= 2;
		}
		return true;
	}

	@Override
	public boolean stopCellEditing() {
		try {
			if (table instanceof CassavaGrid) {
				CassavaGrid grid = (CassavaGrid) table;
				grid.getCommandStack().run(
						new CellChangeCommand(grid.getTableModel(), textArea
								.getText(), row, column));
			}
		} catch (CommandException ex) {
			Log.error(ex);
		}
		return super.stopCellEditing();
	}
}
