package net.asukaze.cassava.macro;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.macro.value.Reference;

public class Environment {
	private static final String EMPTY = ""; //$NON-NLS-1$
	private static final String FUNCTION_ARGCOUNT_SEPARATOR = "/"; //$NON-NLS-1$
	private static final String FUNCTION_FILEID_SEPARATOR = "."; //$NON-NLS-1$
	private static final String CURRENT_FILEID_VAR = "$fileid"; //$NON-NLS-1$
	private static final Pattern NUM_PATTERN = Pattern
			.compile("\\s*[\\+\\-]?(\\d+(\\.\\d*)?|\\.\\d*)([Ee][\\+\\-]?\\d*)?\\s*"); //$NON-NLS-1$

	private ICassavaTableModel model;
	private ICassavaViewModel viewModel;
	private List<Map<String, Object>> varsList;
	private Map<String, UserFunction> functions;

	public Environment() {
		this.model = null;
		this.viewModel = null;
		this.varsList = new ArrayList<Map<String, Object>>();
		this.functions = new HashMap<String, UserFunction>();
	}

	public Object dereference(Object obj) {
		while (obj instanceof Reference) {
			obj = ((Reference) obj).get(this);
		}
		return obj;
	}

	public Double toDouble(Object obj) {
		obj = dereference(obj);
		if (obj == null) {
			return 0.0;
		} else if (obj instanceof Double) {
			return (Double) obj;
		} else if (obj instanceof Integer) {
			return Double.valueOf((Integer) obj);
		} else {
			String str = obj.toString();
			if (str.length() == 0) {
				return Double.valueOf(0.0);
			}
			try {
				return Double.valueOf(str);
			} catch (NumberFormatException ex) {
				return Double.valueOf(0.0);
			}
		}
	}

  /**
   * l𕶎ɕϊ܂B <br>
   * l̏ꍇAu.0v͏o͂܂B
   * 
   * @param obj
   *          ϊl
   * @return ϊ̕
   * @see net.asukaze.cassava.core.util.ValueUtil#toString(double)
   */
  public String toString(Object obj) {
    if (obj == null) {
      return EMPTY;
    } else if (obj instanceof Reference) {
      return ((Reference) obj).getString(this);
    } else if (obj instanceof Double) {
      String str = BigDecimal.valueOf((Double) obj).toPlainString();
      if (str.endsWith(".0")) { //$NON-NLS-1$
        return str.substring(0, str.length() - 2);
      }
      return str;
    }
    return obj.toString();
  }

	public int toInt(Object obj) {
		obj = dereference(obj);
		if (obj == null) {
			return 0;
		} else if (obj instanceof Integer) {
			return (Integer) obj;
		} else {
			return toDouble(obj).intValue();
		}
	}

	public void setModel(ICassavaTableModel model) {
		this.model = model;
	}

	public ICassavaTableModel getModel() {
		return model;
	}

	public void setViewModel(ICassavaViewModel viewModel) {
		this.viewModel = viewModel;
		initXY();
	}

	public ICassavaViewModel getViewModel() {
		return viewModel;
	}

	public void setVars(Map<String, Object> vars) {
		if (varsList.size() == 0) {
			varsList.add(vars);
		} else {
			varsList.set(varsList.size() - 1, vars);
		}
	}

	private Map<String, Object> getVars() {
		if (varsList.size() == 0) {
			varsList.add(new HashMap<String, Object>());
		}
		return varsList.get(varsList.size() - 1);
	}

	public Object getVar(String name) {
		return getVars().get(name);
	}

	public void putVar(String name, Object value) {
		getVars().put(name, value);
	}

	public String getCurrentFileId() {
		return (String) getVar(CURRENT_FILEID_VAR);
	}

	public void push(String fileid) {
		varsList.add(new HashMap<String, Object>());
		putVar(CURRENT_FILEID_VAR, fileid);
		initXY();
	}

	private void initXY() {
		if (viewModel != null) {
			putVar("x", viewModel.getCurrentCol() + 1); //$NON-NLS-1$
			putVar("y", viewModel.getCurrentRow() + 1); //$NON-NLS-1$
		}
	}

	public void pop() {
		if (varsList.size() > 0) {
			varsList.remove(varsList.size() - 1);
		}
	}

	public void putFunction(String fileid, String funcname, int argCount,
			UserFunction userFunction) {
		if (fileid != null) {
			funcname = fileid + FUNCTION_FILEID_SEPARATOR + funcname;
		}
		functions.put(funcname + FUNCTION_ARGCOUNT_SEPARATOR + argCount,
				userFunction);
	}

	public UserFunction getFunction(String fileid, String funcname, int argCount) {
		if (fileid != null) {
			funcname = fileid + FUNCTION_FILEID_SEPARATOR + funcname;
		}
		return functions.get(funcname + FUNCTION_ARGCOUNT_SEPARATOR + argCount);
	}

	public void setValueAt0(String value, int rowIndex, int columnIndex) {
		if (model == null) {
			return;
		}
		model.setValueAt(value, rowIndex, columnIndex);
	}

	public Object getValueAt0(int rowIndex, int columnIndex) {
		String value = getStringValueAt0(rowIndex, columnIndex);
		if (value == null || value.length() == 0) {
			return null;
		}
		if (NUM_PATTERN.matcher(value).matches()) {
			return Double.valueOf(value);
		} else {
			return value;
		}
	}

  public String getStringValueAt0(int rowIndex, int columnIndex) {
    if (model == null) {
      return null;
    }
    return model.getCalculatedValueAt(rowIndex, columnIndex);
  }
}
