package net.asukaze.cassava.macro.action;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.io.util.CsvReader;
import net.asukaze.cassava.macro.APIFunctionHandler;
import net.asukaze.cassava.macro.FunctionNotFoundException;
import net.asukaze.cassava.macro.Macro;
import net.asukaze.cassava.ui.IApplication;
import net.asukaze.cassava.ui.action.Action;

public class ActionFunctionHandler implements APIFunctionHandler {
	private static final String ACTION_PACKAGE = "net.asukaze.cassava.ui.action."; //$NON-NLS-1$
	private static final String ACTION_CLASS_POSTFIX = "Action"; //$NON-NLS-1$
	private static final String ACTION_ALIAS_FILE_NAME = "action_alias.csv"; //$NON-NLS-1$
	private static final String IGNORE_KEYWORD = "#"; //$NON-NLS-1$

	private static Map<String, String> actionAlias = createActionAlias();

	private IApplication mainFrame;

	public ActionFunctionHandler(IApplication mainFrame) {
		this.mainFrame = mainFrame;
	}

	public Object runAPIFunction(String name, Object[] args, Macro macro) {
		if (args.length != 0) {
			return new FunctionNotFoundException(name, args.length);
		}
		boolean result = innerRunAPIFunction(name, args);
		if (result) {
			return null;
		} else {
			return new FunctionNotFoundException(name, args.length);
		}
	}

	public boolean innerRunAPIFunction(String name, Object[] args) {
		try {
			if (mainFrame == null) {
				return false;
			}
			String className = getActionClassName(name);
			Class<?> clazz = Class.forName(className);
			Action action = mainFrame.getAction(clazz);
			if (action == null) {
				return false;
			}
			action.actionPerformed(null);
			return true;
		} catch (ClassNotFoundException ex) {
			return false;
		}
	}

	private static String getActionClassName(String name) {
		if (actionAlias.containsKey(name)) {
			return actionAlias.get(name);
		} else {
			return ACTION_PACKAGE + name + ACTION_CLASS_POSTFIX;
		}
	}

	private static Map<String, String> createActionAlias() {
		Map<String, String> map = new HashMap<String, String>();
		try {
			InputStream is = ActionFunctionHandler.class
					.getResourceAsStream(ACTION_ALIAS_FILE_NAME);
			CsvReader reader = new CsvReader(DataFormat.RFC4180,
					new InputStreamReader(is, StandardCharsets.UTF_8));
			ICassavaTableModel aliasmodel;
			try {
				aliasmodel = reader.readTable();
			} finally {
				reader.close();
			}

			for (int y = 0; y < aliasmodel.getRowCount(); y++) {
				String name = aliasmodel.getValueAt(y, 0);
				if (name.length() == 0 || name.startsWith(IGNORE_KEYWORD)) {
					continue;
				}
				map.put(name, aliasmodel.getValueAt(y, 1));
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		}
		return map;
	}

	public void close() {
	}
}
