package net.asukaze.cassava.macro.api;

import java.awt.Component;
import java.io.IOException;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import net.asukaze.cassava.macro.APIFunctionHandler;
import net.asukaze.cassava.macro.FunctionNotFoundException;
import net.asukaze.cassava.macro.Macro;
import net.asukaze.cassava.macro.Messages;

public class GUIMessageBoxFunctionHandler implements APIFunctionHandler {
	private static final String EMPTY = ""; //$NON-NLS-1$

	private Component dialogParent;

	public GUIMessageBoxFunctionHandler(Component dialogParent) {
		this.dialogParent = dialogParent;
	}

	public Object runAPIFunction(String name, Object[] args, Macro macro)
			throws IOException {
		int n = args.length;
		if ("MessageBox".equals(name) && n <= 3) { //$NON-NLS-1$
			return showMessageBox(args, macro);
		} else if ("InputBox".equals(name) && n <= 3) { //$NON-NLS-1$
			return showInputBox(args, macro);
		}
		return new FunctionNotFoundException(name, n);
	}

	private Integer showMessageBox(Object[] args, Macro macro) {
		if (args.length == 0) {
			return showMessageBox(Messages.getString(Messages.BREAK_POINT),
					Messages.getString(Messages.DIALOG_TITLE),
					JOptionPane.DEFAULT_OPTION);
		} else if (args.length == 1) {
			return showMessageBox(macro.toString(args[0]), Messages
					.getString(Messages.DIALOG_TITLE),
					JOptionPane.DEFAULT_OPTION);
		} else if (args.length == 2) {
			return showMessageBox(macro.toString(args[0]), macro
					.toString(args[1]), JOptionPane.DEFAULT_OPTION);
		} else {
			return showMessageBox(macro.toString(args[0]), macro
					.toString(args[1]), macro.toInt(args[2]));
		}
	}

	private Integer showMessageBox(String message, String title, int optionType) {
		if (optionType == JOptionPane.DEFAULT_OPTION) {
			JOptionPane pane = new JOptionPane();
			pane.setMessage(message);
			JDialog dialog = pane.createDialog(dialogParent, title);
			dialog.setVisible(true);
			return null;
		} else {
			int result = JOptionPane.showConfirmDialog(dialogParent, message,
					title, optionType);
			return Integer.valueOf(result);
		}
	}

	private String showInputBox(Object[] args, Macro macro) {
		if (args.length == 0) {
			return showInputBox(EMPTY, Messages
					.getString(Messages.DIALOG_TITLE), EMPTY);
		} else if (args.length == 1) {
			return showInputBox(macro.toString(args[0]), Messages
					.getString(Messages.DIALOG_TITLE), EMPTY);
		} else if (args.length == 2) {
			return showInputBox(macro.toString(args[0]), Messages
					.getString(Messages.DIALOG_TITLE), macro.toString(args[1]));
		} else {
			return showInputBox(macro.toString(args[0]), macro
					.toString(args[1]), macro.toString(args[2]));
		}
	}

	private String showInputBox(String message, String title, String def) {
		Object result = JOptionPane.showInputDialog(dialogParent, message,
				title, JOptionPane.QUESTION_MESSAGE, null, null, def);
		if (result != null) {
			return result.toString();
		} else {
			return def;
		}
	}

	public void close() {
	}
}
