package net.asukaze.cassava.macro.value;

import javax.swing.JOptionPane;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.macro.Environment;
import net.asukaze.cassava.macro.MacroException;
import net.asukaze.cassava.macro.Messages;

public class SpecialVarReference implements Reference {
	private static final String COL = "Col"; //$NON-NLS-1$
	private static final String ROW = "Row"; //$NON-NLS-1$
	private static final String RIGHT = "Right"; //$NON-NLS-1$
	private static final String BOTTOM = "Bottom"; //$NON-NLS-1$
	private static final String SELLEFT = "SelLeft"; //$NON-NLS-1$
	private static final String SELTOP = "SelTop"; //$NON-NLS-1$
	private static final String SELRIGHT = "SelRight"; //$NON-NLS-1$
	private static final String SELBOTTOM = "SelBottom"; //$NON-NLS-1$
	private static final String TRUE = "true"; //$NON-NLS-1$
	private static final String FALSE = "false"; //$NON-NLS-1$
	private static final String NULL = "null"; //$NON-NLS-1$
	private static final Object MB_ABORTRETRYIGNORE = "MB_ABORTRETRYIGNORE"; //$NON-NLS-1$
	private static final Object MB_OK = "MB_OK"; //$NON-NLS-1$
	private static final Object MB_OKCANCEL = "MB_OKCANCEL"; //$NON-NLS-1$
	private static final Object MB_RETRYCANCEL = "MB_RETRYCANCEL"; //$NON-NLS-1$
	private static final Object MB_YESNO = "MB_YESNO"; //$NON-NLS-1$
	private static final Object MB_YESNOCANCEL = "MB_YESNOCANCEL"; //$NON-NLS-1$
	private static final Object IDABORT = "IDABORT"; //$NON-NLS-1$
	private static final Object IDCANCEL = "IDCANCEL"; //$NON-NLS-1$
	private static final Object IDIGNORE = "IDIGNORE"; //$NON-NLS-1$
	private static final Object IDNO = "IDNO"; //$NON-NLS-1$
	private static final Object IDOK = "IDOK"; //$NON-NLS-1$
	private static final Object IDRETRY = "IDRETRY"; //$NON-NLS-1$
	private static final Object IDYES = "IDYES"; //$NON-NLS-1$

	private String name;

	public SpecialVarReference(String name) {
		this.name = name;
	}

	public Object get(Environment environment) {
		if (TRUE.equalsIgnoreCase(name)) {
			return Integer.valueOf(1);
		} else if (FALSE.equalsIgnoreCase(name)) {
			return Integer.valueOf(0);
		} else if (NULL.equalsIgnoreCase(name)) {
			return null;
		} else if (MB_ABORTRETRYIGNORE.equals(name)) {
			return JOptionPane.YES_NO_CANCEL_OPTION;
		} else if (MB_OK.equals(name)) {
			return JOptionPane.DEFAULT_OPTION;
		} else if (MB_OKCANCEL.equals(name)) {
			return JOptionPane.OK_CANCEL_OPTION;
		} else if (MB_RETRYCANCEL.equals(name)) {
			return JOptionPane.OK_CANCEL_OPTION;
		} else if (MB_YESNO.equals(name)) {
			return JOptionPane.YES_NO_OPTION;
		} else if (MB_YESNOCANCEL.equals(name)) {
			return JOptionPane.YES_NO_CANCEL_OPTION;
		} else if (IDABORT.equals(name)) {
			return JOptionPane.CANCEL_OPTION;
		} else if (IDCANCEL.equals(name)) {
			return JOptionPane.CANCEL_OPTION;
		} else if (IDIGNORE.equals(name)) {
			return JOptionPane.NO_OPTION;
		} else if (IDNO.equals(name)) {
			return JOptionPane.NO_OPTION;
		} else if (IDOK.equals(name)) {
			return JOptionPane.OK_OPTION;
		} else if (IDRETRY.equals(name)) {
			return JOptionPane.YES_OPTION;
		} else if (IDYES.equals(name)) {
			return JOptionPane.YES_OPTION;
		}
		ICassavaTableModel model = environment.getModel();
		ICassavaViewModel viewModel = environment.getViewModel();
		if (model == null || viewModel == null) {
			return environment.getVar(name);
		}
		if (COL.equals(name)) {
			return Integer.valueOf(viewModel.getCurrentCol());
		} else if (ROW.equals(name)) {
			return Integer.valueOf(viewModel.getCurrentRow());
		} else if (RIGHT.equals(name)) {
			return Integer.valueOf(model.getColumnCount());
		} else if (BOTTOM.equals(name)) {
			return Integer.valueOf(model.getRowCount());
		} else if (SELLEFT.equals(name)) {
			return Integer.valueOf(viewModel.getSelection().left);
		} else if (SELTOP.equals(name)) {
			return Integer.valueOf(viewModel.getSelection().top);
		} else if (SELRIGHT.equals(name)) {
			return Integer.valueOf(viewModel.getSelection().right);
		} else if (SELBOTTOM.equals(name)) {
			return Integer.valueOf(viewModel.getSelection().bottom);
		} else {
			throw new MacroException(Messages
					.getString(Messages.SPECIAL_VAR_NOT_FOUND)
					+ name);
		}
	}

  public String getString(Environment environment) {
    Object value = get(environment);
    return environment.toString(value);
  }

	public void set(Object value, Environment environment) {
		ICassavaTableModel model = environment.getModel();
		ICassavaViewModel viewModel = environment.getViewModel();

		if (model == null || viewModel == null) {
			environment.putVar(name, value);
		}
		int x = environment.toInt(value);
		if (COL.equals(name)) {
			viewModel.setCurrentCol(x);
		} else if (ROW.equals(name)) {
			viewModel.setCurrentRow(x);
		} else if (RIGHT.equals(name)) {
			model.setColumnCount(x);
		} else if (BOTTOM.equals(name)) {
			model.setRowCount(x);
		} else if (SELLEFT.equals(name)) {
			Selection sel = viewModel.getSelection();
			int r = Math.max(x, sel.right);
			viewModel.setSelection(new Selection(x, sel.top, r, sel.bottom));
		} else if (SELTOP.equals(name)) {
			Selection sel = viewModel.getSelection();
			int b = Math.max(x, sel.bottom);
			viewModel.setSelection(new Selection(sel.left, x, sel.right, b));
		} else if (SELRIGHT.equals(name)) {
			Selection sel = viewModel.getSelection();
			int l = Math.min(x, sel.left);
			viewModel.setSelection(new Selection(l, sel.top, x, sel.bottom));
		} else if (SELBOTTOM.equals(name)) {
			Selection sel = viewModel.getSelection();
			int t = Math.min(x, sel.top);
			viewModel.setSelection(new Selection(sel.left, t, sel.right, x));
		} else {
			throw new MacroException(Messages
					.getString(Messages.SPECIAL_VAR_NOT_FOUND)
					+ name);
		}
	}

}
