package net.asukaze.cassava.macro;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.StringWriter;

import net.asukaze.cassava.macro.api.ExportWriteFunctionHandler;
import net.asukaze.cassava.macro.parser.ParseException;

import org.junit.Before;
import org.junit.Test;

public class CalendarTest {
	public static final String NL = "\r\n"; //$NON-NLS-1$

	private Macro macro;

	@Before
	public void setUp() {
		macro = new Macro();
	}

	public String run(String input) throws ParseException, IOException {
		StringWriter sw = new StringWriter();
		macro.registerAPIFunctionHandler(new ExportWriteFunctionHandler(sw));
		macro.run(new ByteArrayInputStream(input.getBytes()));
		return sw.toString();
	}

	public Object runExpr(String input) throws ParseException, IOException {
		return macro.runExpr(new ByteArrayInputStream(input.getBytes()));
	}

	@Test
	public void testDiv() throws ParseException, IOException {
		Object result = runExpr("Calendar.div(22,7)"); //$NON-NLS-1$
		assertEquals("3", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void testZeller() throws ParseException, IOException {
		Object result = runExpr("Calendar.zeller(2008,5,10)"); //$NON-NLS-1$
		assertEquals("6", macro.toString(result)); //$NON-NLS-1$
	}

	@Test
	public void testFirstWeek() throws ParseException, IOException {
		Object result = runExpr("Calendar.firstweek(2008,5,0)"); //$NON-NLS-1$
		assertEquals("", macro.toString(result)); //$NON-NLS-1$
		result = runExpr("Calendar.firstweek(2008,5,1)"); //$NON-NLS-1$
		assertEquals("1", macro.toString(result)); //$NON-NLS-1$
	}

	@Test
	public void testIsLeapYear() throws ParseException, IOException {
		Object result = runExpr("Calendar.isLeapYear(2000)"); //$NON-NLS-1$
		assertEquals("1", macro.toString(result)); //$NON-NLS-1$
	}

	@Test
	public void testBody() throws ParseException, IOException {
		Object result = runExpr("Calendar.body(2008,2,29)"); //$NON-NLS-1$
		assertEquals("29", macro.toString(result)); //$NON-NLS-1$
		result = runExpr("Calendar.body(2009,2,29)"); //$NON-NLS-1$
		assertEquals("", macro.toString(result)); //$NON-NLS-1$
	}
}
