package net.asukaze.cassava.macro.cui;

import static org.junit.Assert.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PipedReader;
import java.io.PipedWriter;
import java.io.PrintStream;
import java.io.Writer;
import java.lang.reflect.Field;

import net.asukaze.cassava.core.model.CassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaTableModel;

import org.junit.Before;
import org.junit.Test;

public class InteractiveTest {
	private ICassavaTableModel model;
	private Interactive interactive;
	private Writer inputWriter;
	private PrintStream outputWriter;
	private BufferedReader outputReader;

	@Before
	public void setUp() throws Exception {
		PipedWriter inputWriter = new PipedWriter();
		PipedReader inputReader = new PipedReader();
		inputWriter.connect(inputReader);

		PipedOutputStream outputWriter = new PipedOutputStream();
		PipedInputStream outputReader = new PipedInputStream();
		outputWriter.connect(outputReader);

		this.model = new CassavaTableModel();
		this.interactive = new Interactive(model, null, null);
		this.inputWriter = inputWriter;
		this.outputWriter = new PrintStream(outputWriter);
		this.outputReader = new BufferedReader(new InputStreamReader(
				outputReader));

		Field readerField = Interactive.class.getDeclaredField("in");
		readerField.setAccessible(true);
		readerField.set(interactive, new BufferedReader(inputReader));

		Field writerField = Interactive.class.getDeclaredField("out");
		writerField.setAccessible(true);
		writerField.set(interactive, this.outputWriter);

		Field cuiApiField = Interactive.class.getDeclaredField("cuiApi");
		cuiApiField.setAccessible(true);
		Object cuiApi = cuiApiField.get(interactive);

		Field writerField2 = InteractiveFunctionHandler.class.getDeclaredField("out");
		writerField2.setAccessible(true);
		writerField2.set(cuiApi, this.outputWriter);
	}

	private void runThread() {
		new Thread() {
			@Override
			public void run() {
				try {
					interactive.run();
				} catch (IOException ex) {
					ex.printStackTrace();
				}
			}
		}.start();
	}

	private void inputln(String string) throws IOException {
		inputWriter.write(string);
		inputWriter.write("\r\n");
		inputWriter.flush();
	}

	@Test
	public void testSimpleStatement() throws Exception {
		inputln("1+2*3;");
		runThread();
		String out = outputReader.readLine();
		assertEquals("> 7", out);
	}

	@Test
	public void testSimpleExpr() throws Exception {
		inputln("2+3*4;");
		runThread();
		String out = outputReader.readLine();
		assertEquals("> 14", out);
	}

	@Test
	public void testOneLineComment() throws Exception {
		inputln("5+6*7; // comments { ");
		runThread();
		String out = outputReader.readLine();
		assertEquals("> 47", out);
	}

	@Test
	public void testList0_001() throws Exception {
		inputln("list");
		runThread();
		assertEquals("> +-+", outputReader.readLine());
		assertEquals("| |", outputReader.readLine());
		assertEquals("+-+", outputReader.readLine());
	}

	@Test
	public void testList0_002() throws Exception {
		inputln("[2,2] = \"AAA\";");
		inputln("list");
		runThread();
		assertEquals("> Ok.", outputReader.readLine());
		assertEquals("> +-+---+", outputReader.readLine());
		assertEquals("| |   |", outputReader.readLine());
		assertEquals("| |AAA|", outputReader.readLine());
		assertEquals("+-+---+", outputReader.readLine());
	}
}
